@@ -12,7 +12,7 @@ from pymatgen.entries.compatibility import MaterialsProject2020Compatibility
 from pymatgen.entries.computed_entries import ComputedStructureEntry
 
 from emmet.builders.utils import chemsys_permutations
-from emmet.core.thermo import ThermoDoc
+from emmet.core.thermo import ThermoDoc, PhaseDiagramDoc
 from emmet.core.utils import jsanitize
 
 
@@ -21,6 +21,7 @@ class ThermoBuilder(Builder):
         self,
         materials: Store,
         thermo: Store,
+        phase_diagram: Optional[Store] = None,
         oxidation_states: Optional[Store] = None,
         query: Optional[Dict] = None,
         compatibility=None,
@@ -34,6 +35,8 @@ class ThermoBuilder(Builder):
             materials (Store): Store of materials documents
             thermo (Store): Store of thermodynamic data such as formation
                 energy and decomposition pathway
+            phase_diagram (Store): Store of phase diagram data for each unique chemical system
+            oxidation_states (Store): Store of oxidation state data to use in correction scheme application
             query (dict): dictionary to limit materials to be analyzed
             compatibility (PymatgenCompatability): Compatability module
                 to ensure energies are compatible
@@ -42,8 +45,13 @@ class ThermoBuilder(Builder):
         self.materials = materials
         self.thermo = thermo
         self.query = query if query else {}
-        self.compatibility = compatibility if compatibility else MaterialsProject2020Compatibility("Advanced")
+        self.compatibility = (
+            compatibility
+            if compatibility
+            else MaterialsProject2020Compatibility("Advanced")
+        )
         self.oxidation_states = oxidation_states
+        self.phase_diagram = phase_diagram
         self._completed_tasks: Set[str] = set()
         self._entries_cache: Dict[str, List[ComputedStructureEntry]] = defaultdict(list)
 
@@ -51,7 +59,11 @@ class ThermoBuilder(Builder):
         if oxidation_states is not None:
             sources.append(oxidation_states)
 
-        super().__init__(sources=sources, targets=[thermo], **kwargs)
+        targets = [thermo]
+        if phase_diagram is not None:
+            targets.append(phase_diagram)
+
+        super().__init__(sources=sources, targets=targets, **kwargs)
 
     def ensure_indexes(self):
         """
@@ -67,6 +79,10 @@ class ThermoBuilder(Builder):
         self.thermo.ensure_index("material_id")
         self.thermo.ensure_index("last_updated")
 
+        # Search index for phase_diagram
+        if self.phase_diagram:
+            self.phase_diagram.ensure_index("chemsys")
+
     def prechunk(self, number_splits: int) -> Iterable[Dict]:
         updated_chemsys = self.get_updated_chemsys()
         new_chemsys = self.get_new_chemsys()
@@ -103,17 +119,25 @@ class ThermoBuilder(Builder):
         # Remove overlapping chemical systems
         processed = set()
         to_process_chemsys = []
-        for chemsys in sorted(updated_chemsys | new_chemsys | affected_chemsys, key=lambda x: len(x), reverse=True,):
+        for chemsys in sorted(
+            updated_chemsys | new_chemsys | affected_chemsys,
+            key=lambda x: len(x),
+            reverse=True,
+        ):
             if chemsys not in processed:
                 processed |= chemsys_permutations(chemsys)
                 to_process_chemsys.append(chemsys)
 
-        self.logger.info(f"Found {len(to_process_chemsys)} chemical systems with new/updated materials to process")
+        self.logger.info(
+            f"Found {len(to_process_chemsys)} chemical systems with new/updated materials to process"
+        )
         self.total = len(to_process_chemsys)
 
         # Yield the chemical systems in order of increasing size
         # Will build them in a similar manner to fast Pourbaix
-        for chemsys in sorted(to_process_chemsys, key=lambda x: len(x.split("-")), reverse=True):
+        for chemsys in sorted(
+            to_process_chemsys, key=lambda x: len(x.split("-")), reverse=False
+        ):
             entries = self.get_entries(chemsys)
             yield entries
 
@@ -124,48 +148,65 @@ class ThermoBuilder(Builder):
 
         entries = [ComputedStructureEntry.from_dict(entry) for entry in item]
         # determine chemsys
-        elements = sorted(set([el.symbol for e in entries for el in e.composition.elements]))
+        elements = sorted(
+            set([el.symbol for e in entries for el in e.composition.elements])
+        )
         chemsys = "-".join(elements)
 
         self.logger.debug(f"Processing {len(entries)} entries for {chemsys}")
 
-        material_entries: Dict[str, Dict[str, ComputedStructureEntry]] = defaultdict(dict)
+        material_entries: Dict[str, Dict[str, ComputedStructureEntry]] = defaultdict(
+            dict
+        )
         pd_entries = []
         for entry in entries:
             material_entries[entry.entry_id][entry.data["run_type"]] = entry
 
         with warnings.catch_warnings():
-            warnings.filterwarnings("ignore", message="Failed to guess oxidation states.*")
+            warnings.filterwarnings(
+                "ignore", message="Failed to guess oxidation states.*"
+            )
             pd_entries = self.compatibility.process_entries(entries)
         self.logger.debug(f"{len(pd_entries)} remain in {chemsys} after filtering")
 
         try:
-            docs = ThermoDoc.from_entries(pd_entries, deprecated=False)
+            docs, pd = ThermoDoc.from_entries(pd_entries, deprecated=False)
             for doc in docs:
                 doc.entries = material_entries[doc.material_id]
                 doc.entry_types = list(material_entries[doc.material_id].keys())
 
+            pd_doc = PhaseDiagramDoc(chemsys=chemsys, phase_diagram=pd)
+
         except PhaseDiagramError as p:
             elsyms = []
             for e in entries:
                 elsyms.extend([el.symbol for el in e.composition.elements])
 
-            self.logger.warning(f"Phase diagram error in chemsys {'-'.join(sorted(set(elsyms)))}: {p}")
+            self.logger.warning(
+                f"Phase diagram error in chemsys {'-'.join(sorted(set(elsyms)))}: {p}"
+            )
             return []
         except Exception as e:
-            self.logger.error(f"Got unexpected error while processing {[ent_.entry_id for ent_ in entries]}: {e}")
+            self.logger.error(
+                f"Got unexpected error while processing {[ent_.entry_id for ent_ in entries]}: {e}"
+            )
             return []
 
-        return jsanitize([d.dict() for d in docs], allow_bson=True)
+        return jsanitize(
+            [{**doc.dict(), "phase_diagram": pd_doc.dict()} for d in docs],
+            allow_bson=True,
+        )
 
     def update_targets(self, items):
         """
-        Inserts the thermo docs into the thermo collection
+        Inserts the thermo and phase diagram docs into the thermo collection
         Args:
-            items ([[dict]]): a list of list of thermo dictionaries to update
+            items ([[tuple(dict,dict)]]): a list of list of thermo dictionaries to update
         """
+
         # flatten out lists
         items = list(filter(None, chain.from_iterable(items)))
+
         # Check if already updated this run
         items = [i for i in items if i["material_id"] not in self._completed_tasks]
 
@@ -173,13 +214,19 @@ class ThermoBuilder(Builder):
 
         for item in items:
             if isinstance(item["last_updated"], dict):
-                item["last_updated"] = MontyDecoder().process_decoded(item["last_updated"])
+                item["last_updated"] = MontyDecoder().process_decoded(
+                    item["last_updated"]
+                )
+
+            if "phase_diagram" in item:
+                pd_doc = item.pop("phase_diagram")
+                self.phase_diagram.update(pd_doc)
 
         if len(items) > 0:
             self.logger.info(f"Updating {len(items)} thermo documents")
             self.thermo.update(docs=items, key=["material_id"])
         else:
-            self.logger.info("No items to update")
+            self.logger.info("No thermo items to update")
 
     def get_entries(self, chemsys: str) -> List[Dict]:
         """
@@ -195,16 +242,26 @@ class ThermoBuilder(Builder):
         all_chemsys = chemsys_permutations(chemsys)
         cached_chemsys = all_chemsys & set(self._entries_cache.keys())
         query_chemsys = all_chemsys - cached_chemsys
-        all_entries = list(chain.from_iterable(self._entries_cache[c] for c in cached_chemsys))
+        all_entries = list(
+            chain.from_iterable(self._entries_cache[c] for c in cached_chemsys)
+        )
 
-        self.logger.debug(f"Getting {len(cached_chemsys)} sub-chemsys from cache for {chemsys}")
-        self.logger.debug(f"Getting {len(query_chemsys)} sub-chemsys from DB for {chemsys}")
+        self.logger.debug(
+            f"Getting {len(cached_chemsys)} sub-chemsys from cache for {chemsys}"
+        )
+        self.logger.debug(
+            f"Getting {len(query_chemsys)} sub-chemsys from DB for {chemsys}"
+        )
 
         # Second grab the materials docs
         new_q = dict(self.query)
         new_q["chemsys"] = {"$in": list(query_chemsys)}
         new_q["deprecated"] = False
-        materials_docs = list(self.materials.query(criteria=new_q, properties=["material_id", "entries", "deprecated"]))
+        materials_docs = list(
+            self.materials.query(
+                criteria=new_q, properties=["material_id", "entries", "deprecated"]
+            )
+        )
 
         # Get Oxidation state data for each material
         oxi_states_data = {}
@@ -214,7 +271,10 @@ class ThermoBuilder(Builder):
                 d["material_id"]: d.get("average_oxidation_states", {})
                 for d in self.oxidation_states.query(
                     properties=["material_id", "average_oxidation_states"],
-                    criteria={"material_id": {"$in": material_ids}, "state": "successful"},
+                    criteria={
+                        "material_id": {"$in": material_ids},
+                        "state": "successful",
+                    },
                 )
             }
 
@@ -225,7 +285,9 @@ class ThermoBuilder(Builder):
         # Convert the entries into ComputedEntries and store
         for doc in materials_docs:
             for r_type, entry_dict in doc.get("entries", {}).items():
-                entry_dict["data"]["oxidation_states"] = oxi_states_data.get(entry_dict["entry_id"], {})
+                entry_dict["data"]["oxidation_states"] = oxi_states_data.get(
+                    entry_dict["entry_id"], {}
+                )
                 entry_dict["data"]["run_type"] = r_type
                 elsyms = sorted(set([el for el in entry_dict["composition"]]))
                 self._entries_cache["-".join(elsyms)].append(entry_dict)
@@ -240,7 +302,9 @@ class ThermoBuilder(Builder):
 
         updated_mats = self.thermo.newer_in(self.materials, criteria=self.query)
         updated_chemsys = set(
-            self.materials.distinct("chemsys", {"material_id": {"$in": list(updated_mats)}, **self.query})
+            self.materials.distinct(
+                "chemsys", {"material_id": {"$in": list(updated_mats)}, **self.query}
+            )
         )
         self.logger.debug(f"Found {len(updated_chemsys)} updated chemical systems")
 
@@ -264,7 +328,9 @@ class ThermoBuilder(Builder):
         # First get all chemsys with any of the elements we've marked
         affected_chemsys = set()
         affected_els = list({el for c in chemical_systems for el in c.split("-")})
-        possible_affected_chemsys = self.materials.distinct("chemsys", {"elements": {"$in": affected_els}})
+        possible_affected_chemsys = self.materials.distinct(
+            "chemsys", {"elements": {"$in": affected_els}}
+        )
 
         sub_chemsys = defaultdict(list)
         # Build a dictionary mapping sub_chemsys to all super_chemsys
@@ -276,6 +342,8 @@ class ThermoBuilder(Builder):
         for chemsys in chemical_systems:
             affected_chemsys |= set(sub_chemsys[chemsys])
 
-        self.logger.debug(f"Found {len(affected_chemsys)} chemical systems affected by this build")
+        self.logger.debug(
+            f"Found {len(affected_chemsys)} chemical systems affected by this build"
+        )
 
         return affected_chemsys
